<?php
/*
Plugin Name: 		OxaPay - Crypto Payment Gateway
Plugin URI: 		https://app.oxapay.com/signin
Description:		OxaPay is a leading crypto payment gateway that enables businesses to accept payments in various cryptocurrencies securely.
Version: 			1.0.0
Author: 			OxaPay.com
Author URI: 		https://OxaPay.com
License: 			GPLv2
License URI: 		http://www.gnu.org/licenses/gpl-2.0.html
*/

if (!defined('ABSPATH')) exit;

require_once plugin_dir_path(__FILE__) . 'vendor/autoload.php';

if (!class_exists('Easy_Digital_Downloads') && !class_exists('Edd_Gateway_OxaPay')) return;

DEFINE('OXAPAY', "edd-oxapay");

class Edd_Gateway_OxaPay{
    const SETTING_NAME_MERCHANT = 'oxapay_merchant_key';
    const SETTING_LIFETIME = 'lifetime';
    const SETTING_BUTTON_TITLE = 'title';

    public function __construct()
    {
        $this->method_title = __('OxaPay', OXAPAY);
        $this->method_description = "<a target='_blank' href='https://oxapay.com'>" . __('Plugin Homepage', OXAPAY) . "</a><br/>";
        $this->has_fields = false;

        add_filter('edd_settings_gateways', array($this, 'init_form_fields'));
        add_filter('edd_payment_gateways', array($this, 'register_gateway'));
        add_filter('edd_settings_sections_gateways', array($this, 'register_gateway_section'));
        add_filter('edd_accepted_payment_icons', array($this, 'add_icon'));
        add_action('edd_oxapay_cc_form', '__return_false');
        add_action('init', array($this, 'process_ipn'));
        add_action('edd_gateway_oxapay', array($this, 'process_payment'));

        return true;
    }

    public function register_gateway($gateways)
    {
        $gateways['oxapay'] = array(
            'admin_label' => 'OxaPay',
            'checkout_label' => edd_get_option(self::SETTING_BUTTON_TITLE, __('OxaPay', OXAPAY)),
            'icons' => array(
                plugins_url('./oxapay.png', __FILE__),
            ),
        );

        return $gateways;
    }

    public function register_gateway_section(array $gateway_sections)
    {
        $gateway_sections['oxapay'] = __('OxaPay', OXAPAY);

        return $gateway_sections;
    }

    public function add_icon(array $icons)
    {
        $icon_url = plugins_url('./oxapay.png', __FILE__);
        $button_title = edd_get_option(self::SETTING_BUTTON_TITLE, __('OxaPay', OXAPAY));
        $icons[$icon_url] = $button_title;
        return $icons;
    }

    public function init_form_fields($settings)
    {

        $oxapay_settings = array(
            'oxapay' => array(
                'id' => 'oxapay',
                'name' => '<h3>' . __('OxaPay', OXAPAY) . '</h3><!--' . print_r($settings, true) . ' -->',
                'desc' => "<p>OxaPay is a leading crypto payment gateway that enables businesses to accept payments in various cryptocurrencies securely.</p>",
                'type' => 'descriptive_text',
            ),
            'merchant_api' => array(
                'id' => self::SETTING_NAME_MERCHANT,
                'name' => __('Merchant API Key', OXAPAY),
                'desc' => __('You can find your OxaPay Merchant Key in the settings of your personal account. Read more <a href="https://docs.oxapay.com/integrations/merchant-api">here</a>.', OXAPAY),
                'type' => 'text',
            ),
            'lifetime' => array(
                'id' => self::SETTING_LIFETIME,
                'name' => __('Lifetime', OXAPAY),
                'desc' => __('Set the expiration time for the payment link.', OXAPAY),
                'type' => 'select',
                'options' => array(
                    '30' => __('30 mins', OXAPAY),
                    '60' => __('60 mins', OXAPAY),
                    '90' => __('90 mins', OXAPAY),
                    '120' => __('120 mins', OXAPAY))
            ),
            'button_title' => array(
                'id' => self::SETTING_BUTTON_TITLE,
                'name' => __('Title', OXAPAY),
                'desc' => __('Button title in card', OXAPAY),
                'type' => 'text'
            ),
        );

        $gateway_settings = $settings;
        $gateway_settings['oxapay'] = $oxapay_settings;

        return $gateway_settings;
    }

    function process_payment($purchase_data)
    {
        $errors = edd_get_errors();
        $amount = edd_sanitize_amount($purchase_data['price']);

        if (!$errors) {
            $payment_data = array(
                'price' => $amount,
                'date' => $purchase_data['date'],
                'user_email' => $purchase_data['user_email'],
                'purchase_key' => $purchase_data['purchase_key'],
                'currency' => edd_get_currency(),
                'downloads' => $purchase_data['downloads'],
                'cart_details' => $purchase_data['cart_details'],
                'user_info' => $purchase_data['user_info'],
                'status' => 'New'
            );

            $order_id = edd_insert_payment($payment_data);
            if (!$order_id) {
                edd_set_error('authorize_error', __('Error: Your payment could not be recorded. Please try again.', OXAPAY));
                edd_send_back_to_checkout('?payment-mode=' . $purchase_data['post_data']['edd-gateway']);
                return;
            }

            $additionData = ['order_id' => $order_id];
            try {
                $data = [
                    'merchant' => edd_get_option(self::SETTING_NAME_MERCHANT, 'sandbox'),
                    'amount' => $amount,
                    'currency' => edd_get_currency(),
                    'orderId' => $order_id,
                    'returnUrl' => edd_get_success_page_uri() . '?order_id=' . $order_id,
                    'callbackUrl' => trailingslashit(home_url()) . '?gateway=OxaPay',
                    'lifeTime' => edd_get_option(self::SETTING_LIFETIME, '60'),
                ];
                $url = 'https://api.oxapay.com/merchants/request';
                $options = array(
                    'http' => array(
                        'header' => 'Content-Type: application/json',
                        'method'  => 'POST',
                        'content' => json_encode($data),
                    ),
                );
                $context  = stream_context_create($options);
                $response = file_get_contents($url, false, $context);
                $result = json_decode($response);
            } 
            catch (Exception $e) {
                edd_record_gateway_error(__('OxaPay error', OXAPAY), print_r($e, true), 0);
                edd_set_error('card_declined', __('Error: OxaPay error create payment ', OXAPAY) . $e->getMessage());
                edd_send_back_to_checkout('?payment-mode=' . $purchase_data['post_data']['edd-gateway']);
                return;
            }

            if ($result->result == 100) {
                $note = __('OxaPay Invoice ID', OXAPAY) . ': <a target = "_blank" href = "' . $result->payLink . '">' .$result->trackId. '</a> ' . __('is processing', OXAPAY);
                edd_insert_payment_note($order_id, $note);
                edd_empty_cart();
                wp_redirect($result->payLink);
            } else {
                edd_update_payment_status($order_id, 'failed');
                edd_get_confirmation_page_uri();
            }

        } else {
            edd_send_back_to_checkout('?payment-mode=' . $purchase_data['post_data']['edd-gateway']);
        }

    }

    public static function convertToWPStatus($status)
    {
        switch ($status) {
            case 'Waiting':
                $result = 'pending';
                break;

            case 'Confirming':
                $result = 'processing';
                break;

            case 'Failed':
            case 'Expired':
                $result = 'failed';
                break;
            case 'Paid':
                $result = 'completed';
                break;
        }
        return $result;
    }

    public function process_ipn()
    {
        if (!isset($_GET['gateway'])) {
            return;
        }
        $postData = file_get_contents('php://input');
        $data = json_decode($postData, true);
        $apiSecretKey = edd_get_option(self::SETTING_NAME_MERCHANT, 'sandbox');
        $hmacHeader = $_SERVER['HTTP_HMAC'];
        $calculatedHmac = hash_hmac('sha512', $postData, $apiSecretKey);
        if ($calculatedHmac === $hmacHeader) {
            $orderId = $data['orderId'];
            edd_update_payment_status($orderId , self::convertToWPStatus($data['status']));
            http_response_code(200);
            echo 'OK';
        } else {
            http_response_code(400);
            echo 'Invalid HMAC signature';
        }
    }
}

new Edd_Gateway_OxaPay();
